@@ -7,7 +7,7 @@ use Config;
 use Text::ParseWords qw( shellwords );
 
 # NOTE: Although this module is now distributed with Alien-Build,
-# it should have NO non-perl-core dependencies for all Perls 
+# it should have NO non-perl-core dependencies for all Perls
 # 5.8.1-5.26.0 (as of this writing, and any Perl more recent).
 # You should be able to extract this module from the rest of
 # Alien-Build and use it by itself.
@@ -16,7 +16,7 @@ use Text::ParseWords qw( shellwords );
 # distributed with Alien-Build.
 
 # ABSTRACT: Compiler and linker wrapper for Alien
-our $VERSION = '1.74'; # VERSION
+our $VERSION = '1.93'; # VERSION
 
 
 my @cflags_I;
@@ -109,7 +109,7 @@ sub mb_args
 
 sub _join
 {
-  join ' ', map { s/(\s)/\\$1/g; $_ } @_;
+  join ' ', map { s/(\s)/\\$1/g; $_ } map { "$_" } @_;  ## no critic (ControlStructures::ProhibitMutatingListFunctions)
 }
 
 sub import
@@ -141,18 +141,18 @@ sub import
       $cflags = $alien->cflags;
       $libs   = $alien->libs;
     }
-    
+
     push @cflags_I,     grep  /^-I/, shellwords $cflags;
     push @cflags_other, grep !/^-I/, shellwords $cflags;
-    
+
     push @ldflags_L,     grep  /^-L/,    shellwords $libs;
     push @ldflags_l,     grep  /^-l/,    shellwords $libs;
     push @ldflags_other, grep !/^-[Ll]/, shellwords $libs;
   }
-  
+
   my @cflags_define = grep  /^-D/, @cflags_other;
   my @cflags_other2 = grep !/^-D/, @cflags_other;
-  
+
   @mm = ();
 
   push @mm, INC       => _join @cflags_I                             if @cflags_I;
@@ -166,10 +166,10 @@ sub import
   push @mm, LDFLAGS   => _join(@ldflags) . " $Config{ldflags}"       if @ldflags;
 
   @mb = ();
-  
+
   push @mb, extra_compiler_flags => _join(@cflags_I, @cflags_other);
   push @mb, extra_linker_flags   => _join(@ldflags_l);
-  
+
   if(@ldflags)
   {
     push @mb, config => {
@@ -177,14 +177,14 @@ sub import
       ldflags   => _join(@ldflags) . " $Config{ldflags}",
     },
   }
-  
+
   if($export)
   {
     my $caller = caller;
     no strict 'refs';
     *{"${caller}::cc"} = \&cc;
     *{"${caller}::ld"} = \&ld;
-  }  
+  }
 }
 
 1;
@@ -201,7 +201,7 @@ Alien::Base::Wrapper - Compiler and linker wrapper for Alien
 
 =head1 VERSION
 
-version 1.74
+version 1.93
 
 =head1 SYNOPSIS
 
@@ -249,7 +249,7 @@ From Makefile.PL (dynamic):
      }
    ;
  }
-
+ 
  WriteMakefile(
    'NAME'         => 'Foo::XS',
    'VERSION_FROM' => 'lib/Foo/XS.pm',
